﻿using System.Collections;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.ComponentModel;
using System.Diagnostics;
using System.Linq;
using System.Windows;
using System.Windows.Data;

using CustomControlForXamDataGrid.ViewModel;

using Infragistics.Windows.DataPresenter;

using ViewModels;

namespace CustomControlForXamDataGrid
{
    /// <summary>
    /// Interaction logic for ChannelDataGrid.xaml
    /// </summary>
    public partial class ChannelDataGrid : XamDataGrid
    {
        private bool _isSelectEntireColumn = false;

        private List<ParameterViewModel> _selectedCells=new List<ParameterViewModel>();
      

        #region Dependency Properties

        /// <summary>
        /// Identifies the <c>IsManaged</c> attached dependency property.  This field is read-only.
        /// </summary>
        /// <value>The identifier for the <c>IsManaged</c> attached dependency property.</value>
        public static readonly DependencyProperty ParametersOrderListProperty = DependencyProperty.RegisterAttached(
            "ParametersOrderList",
            typeof(List<string>),
            typeof(ChannelDataGrid),
            new PropertyMetadata(null, OnParametersOrderListPropertyPropertyValueChanged));

        public static readonly DependencyProperty SelectedCellsProperty = DependencyProperty.RegisterAttached(
           "SelectedCells",
           typeof(List<ParameterViewModel>),
           typeof(ChannelDataGrid),
           new PropertyMetadata(null, OnSelectedCellsPropertyPropertyValueChanged));

        private IEnumerable<ChannelViewModel> _data;

        #endregion // Dependency Properties

        /// <summary>
        /// Initializes a new instance of the <see cref="ChannelDataGrid"/> class.
        /// </summary>
        public ChannelDataGrid()
        {
            InitializeComponent();
        }

        /// <summary>
        /// Gets or sets the parameters list.
        /// </summary>
        /// <value>The parameters list.</value>
        public List<string> ParametersOrderList
        {
            get
            {
                return (List<string>)GetValue(ParametersOrderListProperty);
            }

            set
            {
                SetValue(ParametersOrderListProperty, value);
            }
        }

        public List<ParameterViewModel> SelectedCells
        {
            get
            {
                //return (List<ParameterViewModel>)GetValue(SelectedCellsProperty);
                return this._selectedCells;
            }

            set
            {
                SetValue(SelectedCellsProperty, value);
            }
        }

        /// <summary>
        /// Called when [parameters list property value changed].
        /// </summary>
        /// <param name="d">The d.</param>
        /// <param name="e">The <see cref="System.Windows.DependencyPropertyChangedEventArgs"/> instance containing the event data.</param>
        private static void OnParametersOrderListPropertyPropertyValueChanged(DependencyObject d, DependencyPropertyChangedEventArgs e)
        {
            var channelDataGrid = (ChannelDataGrid)d;

            for (int index = 0; index < channelDataGrid.ParametersOrderList.Count; index++)
            {
                var parameter = channelDataGrid.ParametersOrderList[index];
                UnboundField parameterField = new UnboundField();
                parameterField.BindingPath = new PropertyPath("Parameters[" + parameter + "].Value", null);
                parameterField.BindingMode = BindingMode.TwoWay;
                parameterField.Name = parameter;
                parameterField.Label = parameter;
                parameterField.DataType = typeof(string);
                parameterField.Settings.CellValuePresenterStyle = CreateCellStryle(parameter);
                
                channelDataGrid.FieldLayouts[0].Fields.Add(parameterField);
            }
        }

        /// <summary>
        /// Creates the na cell stryle.
        /// </summary>
        /// <param name="parameterName">Name of the parameter.</param>
        /// <returns>The style.</returns>
        private static Style CreateCellStryle(string parameterName)
        {
            var cellStryle = new Style { TargetType = typeof(CellValuePresenter) };
            var enableSetter = new Setter(
                IsEnabledProperty,
                new Binding("DataItem.Parameters[" + parameterName + "].IsEnabled")
                {
                    Mode = BindingMode.OneWay
                });

            cellStryle.Setters.Add(enableSetter);

            return cellStryle;
        }

        private static void OnSelectedCellsPropertyPropertyValueChanged(DependencyObject d, DependencyPropertyChangedEventArgs e)
        {
            var channelDataGrid = (ChannelDataGrid)d;
        }

        /// <summary>
        /// Handles the Sorting event of the dataGridCommon control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="Infragistics.Windows.DataPresenter.Events.SortingEventArgs"/> instance containing the event data.</param>
        private void dataGridCommon_Sorting(object sender, Infragistics.Windows.DataPresenter.Events.SortingEventArgs e)
        {
            if (_data == null)
            {
                _data = DataSource as IEnumerable<ChannelViewModel>;
            }
            
            ListSortDirection sortDirection = e.SortDescription.Direction;

            switch (sortDirection)
            {
                case ListSortDirection.Ascending:
                    SortAsc(e.SortDescription.FieldName);
                    break;
                case ListSortDirection.Descending:
                    SortDesc(e.SortDescription.FieldName);
                    break;
            }
        }

        private void SortAsc(string fieldName)
        {
            Stopwatch stopwatch = new Stopwatch();
            stopwatch.Start();

            if (fieldName == "Name")
            {
                DataSource = _data.OrderBy(c => c.Name);

                stopwatch.Stop();
                return;
            }

            IOrderedEnumerable<ChannelViewModel> channelViewModels = _data.OrderBy(c => c.Parameters.First(p => p.Key == fieldName).Value.Value);

            stopwatch.Stop();

            Stopwatch stopwatch1 = new Stopwatch();
            stopwatch1.Start();

            DataSource = channelViewModels;

            stopwatch1.Stop();
        }

        private void SortDesc(string fieldName)
        {
            Stopwatch stopwatch = new Stopwatch();
            stopwatch.Start();

            if (fieldName == "Name")
            {
                DataSource = _data.OrderByDescending(c => c.Name);

                stopwatch.Stop();
                return;
            }

            IOrderedEnumerable<ChannelViewModel> orderByDescending = _data.OrderByDescending(c => c.Parameters.First(p => p.Key == fieldName).Value.Value);

            stopwatch.Stop();

            Stopwatch stopwatch1 = new Stopwatch();
            stopwatch1.Start();

            DataSource = orderByDescending;

            stopwatch1.Stop();
        }

        private void XamDataGrid_PreviewMouseRightButtonDown(object sender, System.Windows.Input.MouseButtonEventArgs e)
        {
            var xamDataGrid1 = sender as XamDataGrid;
            var labelPresenter = Infragistics.Windows.Utilities.GetAncestorFromType(e.OriginalSource as DependencyObject, typeof(LabelPresenter), true);
            if (labelPresenter != null)
            {
                _isSelectEntireColumn = true;

               _selectedCells.Clear();
                xamDataGrid1.SelectedItems.Cells.Clear();
                xamDataGrid1.SelectedItems.Fields.Clear();
                var clickedFieldName = ((DataItemPresenter)labelPresenter).Field.Name;
                var field = xamDataGrid1.FieldLayouts[0].Fields[clickedFieldName];
                xamDataGrid1.SelectedItems.Fields.Add(field);
                
                foreach (DataRecord record in xamDataGrid1.Records)
                {
                    if (record.IsFilteredOut.Equals(true))
                    {
                        continue;
                    }

                    var cellsUnderColumn = CellValuePresenter.FromCell(record.Cells[clickedFieldName]);
                    var vm = record.DataItem as ChannelViewModel;                    
                    if (vm != null)
                    {

                        foreach (var parameterViewModel in vm.Parameters.Values)
                        {
                            if (parameterViewModel.ParameterName == clickedFieldName)
                            {
                                this._selectedCells.Add(parameterViewModel);
                            }
                        }
          
                    }
                }

                SelectedCells = _selectedCells;
                _isSelectEntireColumn = false;
            }
        }

        private void XamDataGrid_SelectedItemsChanged(object sender, Infragistics.Windows.DataPresenter.Events.SelectedItemsChangedEventArgs e)
        {
            var xamDataGrid1 = sender as XamDataGrid;
            if (!_isSelectEntireColumn)
            {
                //_selectedCells = xamDataGrid1.SelectedItems.Cells;
            }   
        }
    }
}
